#!/usr/local/bin/perl -w

use strict;
use warnings;

use CGI;
my $q = new CGI;
use CGI::Cookie;
use URI::Escape;

use Fcntl qw(:flock);

use Phoneme;
use Constraints;





# read cookies
my %cookies = fetch CGI::Cookie;

# sets phonetic alphabet
my $alphabet = $cookies{alphabet} || "";
$alphabet = ($alphabet =~ /^alphabet=([^;]*);/)[0] || "";
$alphabet = "APAU" unless ($alphabet =~ /^[IA]PA[U]?$/);
sub unicode { return (index($alphabet,"U")!=-1) ? "U" : ""; }
setAlphabet($alphabet);


# set CON
my $CON = uri_unescape(($cookies{CON} =~ /^CON=([^;]*);/)[0]) || "";
$CON = "" if ($q->param("keywords") eq "new");

#set inputs
my $inputs = uri_unescape(($cookies{inputs} =~ /^inputs=([^;]*);/)[0]) || "";



# read current CON
my @currCON = split(/,/,$CON);
my %seen = ();
my $showCON;
for (@currCON) {
	if (exists ${CON()}{$_}) {
		$showCON .= "<OPTION VALUE='$_'>" . ${CON()}{$_}->() . "</OPTION>";
		$seen{$_} = 1;
	} else {
		die "The constraint named '$_' caused an error. $!"
	}
}
$showCON =~ s'&#660;'?';  # replace glottal stop with question mark


my $CONtipsScriptStr;
my $unusedConstrains;
for (sort keys %{CON()}) {
	if (exists ${CON()}{$_}) {
		$unusedConstrains .= "<OPTION VALUE='$_'>" . ${CON()}{$_}->() . "</OPTION>" unless $seen{$_};
		$CONtipsScriptStr .=  "CONtips[\"" . $_ . "\"] = \"" . ${CON()}{$_}->() . "<BR>${CONdescription()}{$_}\";\n\t";
	} else {
		die "The constraint named '$_' caused an error. $!"
	}
}
$unusedConstrains =~ s'&#660;'?';  # replace glottal stop with question mark



# read nav bar
my $navStr;
open (NAV, "<navbar.txt") or die "Can't read navbar. $!";
while (<NAV>) {
    $navStr .= $_;
}
close(NAV);

my $menuStr;
open (NAV, "<ranking_menu.txt") or die "Can't read ranking menu. $!";
while (<NAV>) {
    $menuStr .= $_;
}
close(NAV);



# create the body of the page
print $q->header;
print <<"EOF";
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<HTML xmlns="http://www.w3.org/1999/xhtml" lang="en-US" xml:lang="en-US">
<HEAD>
	<TITLE>CCamelOT - An OT-CC implementation</TITLE>
	
	<LINK rel="stylesheet" type="text/css" href="mich.css" />
	<META http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
	
	<SCRIPT src="CONEditor.js" type="text/javascript"></SCRIPT>
	<SCRIPT src="toolbox.js" type="text/javascript"></SCRIPT>
	<SCRIPT type="text/javascript">
		//<![CDATA[
		var CON = "$CON";
		var CONtips = [];
		$CONtipsScriptStr
		//]]>

		function goTransfer(selectName) {
			var temp = (selectName == "currConstraints") ? 0 : 1;
			if (document.conEditor[selectName].options.length) setTimeout("transferConstraints(" + temp + ")",100);
		}

		function showTip(selectName) {
			setTimeout("showTip2('" + selectName + "')",100);
		}
		function showTip2(selectName) {
			if (document.conEditor[selectName].value) {
				var divStr = "";
				divStr = CONtips[document.conEditor[selectName].value];
				document.getElementById("tips").innerHTML = divStr;
			}
		}

		function init() {
			document.getElementById("ranking_tab").className = "navOn";
			if (CON=="") {
				document.getElementById("newTab").className = "menuOn" ;
			} else {
				document.getElementById("editTab").className = "menuOn" ;
			}
		}

	</SCRIPT>
</HEAD>

<BODY LEFTMARGIN=0 TOPMARGIN=0 MARGINWIDTH=0 MARGINHEIGHT=0 onLoad="init()">

$navStr

$menuStr

<DIV CLASS="menuMain" STYLE="padding-top:15px;">

	<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0>
	<FORM NAME="conEditor">
	<TR ALIGN="center" VALIGN="top" STYLE="line-height:190%;">
		<TD>&nbsp;</TD>
		<TD>Current ranking</TD>
		<TD>&nbsp;</TD>
		<TD>Unused contraints</TD>
		<TD>&nbsp;</TD>
	</TR>
	<TR ALIGN="center" VALIGN="top">
		<TD VALIGN="middle" STYLE="padding-right:10px;">
			<A HREF="javascript:rerankConstraints(0,1)"><IMG SRC="images/arrow_uu.gif" WIDTH="24" HEIGHT="27" VSPACE=7 BORDER=0 ALT="Promote selected constraint(s) to the top of the hierarchy"></A><BR>
			<A HREF="javascript:rerankConstraints(0,0)"><IMG SRC="images/arrow_u.gif"  WIDTH="24" HEIGHT="12" VSPACE=7  BORDER=0 ALT="Promote selected constraint(s)"></A><BR>
			<A HREF="javascript:rerankConstraints(1,0)"><IMG SRC="images/arrow_d.gif"  WIDTH="24" HEIGHT="12" VSPACE=7  BORDER=0 ALT="Demote selected constraint(s)"></A><BR>
			<A HREF="javascript:rerankConstraints(1,1)"><IMG SRC="images/arrow_dd.gif" WIDTH="24" HEIGHT="27" VSPACE=7 BORDER=0 ALT="Demote selected constraint(s) to the bottom of the hierarchy"></A><BR>
		</TD>
		<TD>
			<SELECT onClick="javascript:showTip(this.name)" onDblClick="goTransfer(this.name)" ID="currConstraints" NAME="currConstraints"  STYLE='width:200px;' MULTIPLE SIZE=15>$showCON</SELECT>
		</TD>
		<TD VALIGN="middle" NOWRAP><A HREF="javascript:transferConstraints(1)"><IMG SRC="images/arrow_l.gif" WIDTH="12" HEIGHT="24" BORDER=0 HSPACE=5 ALT="Add constraint(s) to current ranking"></A><A HREF="javascript:transferConstraints(0)"><IMG SRC="images/arrow_r.gif" WIDTH="12" HEIGHT="24" HSPACE=5 BORDER=0 ALT="Remove constraint(s) from current ranking"></A></TD>
		<TD>
			<SELECT onClick="javascript:showTip(this.name)" onDblClick="goTransfer(this.name)" ID="unusedConstraints" NAME="unusedConstraints" STYLE='width:200px;' MULTIPLE SIZE=15>$unusedConstrains</SELECT>
		</TD>
		<TD STYLE="padding-left:20px; background: #FFFFFF;">
			<DIV ID="tips"></DIV>
		</TD>
	</TR>
	</FORM>
	</TABLE>

</DIV>



EOF

# read footer
open (FTR, "<footer.txt") or die "Can't read footer. $!";
print while (<FTR>);
close(FTR);

print <<"EOF";



</BODY>
</HTML>

EOF
