#!/usr/bin/perl -w

use strict;
use warnings;

use Constraints;
use Phoneme;

use CGI;
my $q = new CGI;
use CGI::Cookie;

# read cookies
my %cookies = fetch CGI::Cookie;

# sets phonetic alphabet
my $alphabet = $cookies{alphabet} || "";
$alphabet = ($alphabet =~ /^alphabet=([^;]*);/)[0] || "";
$alphabet = "APAU" unless ($alphabet =~ /^[IA]PA[U]?$/);
sub unicode { return (index($alphabet,"U")!=-1) ? "U" : ""; }
setAlphabet($alphabet);


my $CONsep = "<SPAN CLASS='CONsep'><SPAN STYLE='position:relative;left:3px;'>></SPAN><SPAN STYLE='position:relative;left:-3px;'>></SPAN></SPAN>";


# create the body of the page
print $q->header;
print <<"EOF";
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<HTML xmlns="http://www.w3.org/1999/xhtml" lang="en-US" xml:lang="en-US">
<HEAD>
	<TITLE>CCamelOT - An OT-CC implementation</TITLE>
	
	<LINK rel="stylesheet" type="text/css" href="mich.css" />
	<META http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
	<SCRIPT>
		function toggleView(elem) {
			if (document.getElementById) {
				document.getElementById(elem).style.display = (document.getElementById(elem).style.display == "block") ? "none" : "block";
			}
		}
		
		function showDiv() {
			if (location.href.indexOf("#")!=-1) {
				var displayDiv = location.href.substr(location.href.indexOf("#")+1,location.href.length);
				if (document.getElementById(displayDiv)) toggleView(displayDiv);
			}
		}
	
	</SCRIPT>
</HEAD>

<BODY LEFTMARGIN=0 TOPMARGIN=0 MARGINWIDTH=0 MARGINHEIGHT=0 onLoad="showDiv()">

EOF


# read nav bar
open (NAV, "<navbar.txt") or die "Can't read navbar. $!";
print while (<NAV>);
close(NAV);

print <<"EOF";
<DIV CLASS="pagebody">

	<DIV CLASS=title>Help</DIV>
	<BR>

	<A HREF="javascript:toggleView('tour')">A guided tour to CCamelOT</A><BR>
	<BR>
	<DIV CLASS="helpDiv" ID="tour">
		
		<DIV STYLE="font-size:170%;font-weight:bold;">CCamelOT &mdash; The Guided Tour</DIV>

		<OL>

		<LI>
		In CCamelOT, as in classic OT, a derivation starts with an input and a constraint ranking (grammar).<BR>
		Start constructing your input by clicking the "Input" link.<BR>
		<BR>
		<IMG SRC="images/tour1.gif"  WIDTH=818 HEIGHT=218>
		</LI>


		<LI>
		If you have a phonetic font properly installed and selected, you will see something like this. If not, go to the <A HREF="font.cgi">font</A> page.<BR>
		Let's construct the input /ba/, either by typing "ba" or by clicking the "b" button and then the "a" button.<BR>
		<BR>
		<IMG SRC="images/tour2.gif"  WIDTH=818 HEIGHT=582>
		</LI>


		<LI>
		Now that you have an input, you are ready to make a constraint ranking.<BR>
		Start constructing your constraint ranking by clicking the "Ranking" link.<BR>
		<BR>
		<IMG SRC="images/tour3.gif"  WIDTH=818 HEIGHT=203>
		</LI>


		<LI>
		In order to see devoicing of obstruents, let's choose the ranking
		<SPAN STYLE="font-variant:small-caps;">Max$CONsep*Appendix$CONsep*Voice</SPAN>.<BR>
		Your constraint ranking, which is currently empty, is on the left. Available constraints are on the right, with Faithfulness constraints in alphabetical order first, then Markedness constraints in alphabetical order.<BR>
		<BR>
		Find the constraint <SPAN STYLE="font-variant:small-caps;">Max</SPAN> and add it to the your ranking by clicking on it, then clicking on the left arrow (circled in red). You can also add a constraint to your ranking by double-clicking it.<BR>
		<BR>
		<IMG SRC="images/tour4.gif"  WIDTH=818 HEIGHT=563>
		</LI>


		<LI>
		Find <SPAN STYLE="font-variant:small-caps;">*Appendix</SPAN> and
		<SPAN STYLE="font-variant:small-caps;">*Voice</SPAN> and add them to your
		ranking. You can change the ranking of the constraints you have by using
		the up and down arrows (circled in red). For the time being, keep
		<SPAN STYLE="font-variant:small-caps;">Max</SPAN> at the top of your hierarchy.<BR>
		<BR>
		<IMG SRC="images/tour5.gif"  WIDTH=818 HEIGHT=563>
		</LI>


		<LI>
		Now that you have an input and a constraint ranking, you are ready to see the derivation.<BR>
		Click the "Tableau" link in order to run your input through your grammar.<BR>
		<BR>
		As Expected, you get two candidate. The first one is the faithful candidate, which is the most
		harmonic candidate that incurs no faithfulness violations. The second candidate was created in response to
		<SPAN STYLE="font-variant:small-caps;">*Voice</SPAN>. This is the only non-faithful candidate that was generated,
		because there is no other way to improve the faithful candidate with the
		current grammar: A violation of <SPAN STYLE="font-variant:small-caps;">*Voice</SPAN> can be fixed either by devoicing, as in candidate (b), or by deletion of the voiced obstruent,
		but deletion is blocked here by the higher ranking <SPAN STYLE="font-variant:small-caps;">Max</SPAN>.<BR>
		<BR>
		<IMG SRC="images/tour6.gif"  WIDTH=818 HEIGHT=383>
		</LI>


		<LI>
		Let's see what happens if we promote <SPAN STYLE="font-variant:small-caps;">*Voice</SPAN>
		over <SPAN STYLE="font-variant:small-caps;">Max</SPAN>. You could do this by
		clicking the "Ranking" link, but you can also rerank directly from the
		Tableau page with the arrows that are under each constraint's name
		(circled in red). Promote <SPAN STYLE="font-variant:small-caps;">*Voice</SPAN>
		twice, until it's at the top of the hierarchy.<BR>
		<BR>
		<IMG SRC="images/tour7.gif"  WIDTH=818 HEIGHT=383>
		</LI>


		<LI>
		You see that for this grammar, another candidate was created. Since
		<SPAN STYLE="font-variant:small-caps;">*Voice</SPAN> outranks
		<SPAN STYLE="font-variant:small-caps;">Max</SPAN>, deletion of a voiced obstruent
		will improve harmony. The output [pa] is still the winner, since it's more faithful
		than [a], and just as marked under the current grammar.<BR>
		<BR>
		If you want to get an idea of what the correspondence relations in this
		derivation are, check the "Segment indices" box (circled in red). Each segment
		will show its index as a subscript.<BR>
		<BR>
		<IMG SRC="images/tour8.gif"  WIDTH=818 HEIGHT=423>
		</LI>


		<LI>
		When segment indices are visible, it makes sense to look at the LUMs (Localized
		Unfaithful Mappings), or the phonological operations that created the candidates
		that you see. You do this by checking the "LUMs" box (circled in red).<BR>
		You see that candidate (b) was created by deletion of the segment whose index is 1,
		and candidate (c) was created by changing the feature [voice] on the same segment.
		Since by definition no phonological operations were done to the faithful
		candidate (a), it has no LUMs.<BR>
		<BR>
		<IMG SRC="images/tour9.gif"  WIDTH=818 HEIGHT=461>
		</LI>


		<LI>
		The next thing we should look at is the effect of morphological structure on the
		derivation. The do this, we should add a constraint that is sensitive to the morphology,
		like <SPAN STYLE="font-variant:small-caps;">Ident<sub>root</sub></SPAN>(voice).
		Go back to the Ranking page, add this constraint to your ranking, and promote it to
		the top of your hierarchy.<BR>
		<BR>
		<IMG SRC="images/tour10.gif"  WIDTH=818 HEIGHT=567>
		<LI>
		
		
		</LI>
		Next, promote <SPAN STYLE="font-variant:small-caps;">Max</SPAN>
		to the top, so you having the	ranking 
		<SPAN STYLE="font-variant:small-caps;">Max${CONsep}Ident<sub>root</sub></SPAN>(voice)<SPAN STYLE="font-variant:small-caps;">$CONsep*Voice$CONsep*Appendix</SPAN>.<BR>
		<BR>
		Now, let's add the suffix /-di/ to our root /ba/. Go to the input page, and add
		the segments /d/ and /i/ to your input. Then, open the "Morphology" tab (circled in red).<BR>
		<BR>
		<IMG SRC="images/tour11.gif"  WIDTH=818 HEIGHT=519>
		</LI>


		<LI>
		You see that all the segments are currently specified as root segments. To specify
		/-di/ as a suffix, first click the "Affix" button for the /i/, then for the /d/ (circled in red).
		You will not be able to do this in the opposite order, because the root has be
		contiguous <B>in the input</B> (as most theories of morphology assume).<BR>
		<BR>
		<IMG SRC="images/tour12.gif"  WIDTH=818 HEIGHT=519>
		</LI>


		<LI>
		Now let's run this input through the grammar by going back to the tableau page.
		You will see that the root's voiced stop is protected from devoicing by
		<SPAN STYLE="font-variant:small-caps;">Ident<sub>root</sub></SPAN>(voice), but
		there is no faithfulness constraint to protect the suffix.<BR>
		<BR>
		<IMG SRC="images/tour13.gif"  WIDTH=818 HEIGHT=405>
		</LI>
		
		<LI>
		This is the end of the guided tour. You can find more about CCamelOT and the
		way it works further down this page. Enjoy!
		</LI>

		</OL>
	</DIV>

	<A HREF="javascript:toggleView('Symbols')">Notation and symbols</A><BR>
	<BR>
	<DIV CLASS='helpDiv' ID='Symbols'>
		The following symbols are used in CCamelOT:<BR>
		<BR>
		<TABLE CELLPADDING=6 CELLSPACING=0 BORDER=0>
		<TR>
			<TD ALIGN="center"><SPAN STYLE='font-size:130%;'><SPAN CLASS='chain_delimiter'>/</SPAN> <SPAN CLASS='chain_delimiter'>/</SPAN></SPAN></TD>
			<TD>Input, Underlying Representation</TD>
		</TR>
		<TR>
			<TD ALIGN="center"><SPAN STYLE='font-size:130%;'>.</SPAN></TD>
			<TD>Syllable boundary</TD>
		</TR>
		<TR>
			<TD ALIGN="center"><SPAN STYLE='font-size:130%;'>&lt; &gt;</SPAN></TD>
			<TD>Appendix (unsyllabified segments)</TD>
		</TR>
		<TR>
			<TD ALIGN="center"><SPAN STYLE='font-size:130%;'>[ ]</SPAN></TD>
			<TD>Root</TD>
		</TR>
		<TR>
			<TD ALIGN="center"><SPAN STYLE='font-size:130%;'><SPAN CLASS='chain_delimiter'>&lt;</SPAN> <SPAN CLASS='chain_delimiter'>&gt;</SPAN></SPAN></TD>
			<TD>Candidate chain</TD>
		</TR>
		</TABLE>
		<BR>
	</DIV>

	<A HREF="javascript:toggleView('fonts')">Browsers and fonts</A><BR>
	<BR>
	<DIV CLASS="helpDiv" ID="fonts"><A NAME="fonts"></A>
		CCamelOT requires a web-browser that allows cookies and Javascript.<BR>
		<BR>
		
		CCamelOT was tested and found to be fully functional on MS-Explorer and Firefox for Windows and Mac. 
		There are a few known Safari bugs.
		Please let me know if CCamelOT is not fully functional on your favorite browser (but don't forget to install an appropriate font).<BR>
		<BR>
		The phonetic font you should use is the freely downloadable <A HREF="http://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&item_id=DoulosSILfont">Doulos SIL</A> unicode font. 
		You can also use <A HREF="http://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&item_id=Gentium">Gentium</A>, but I find it to be less legible on the screen.
		These unicode fonts are the only fonts that work on Firefox and Safari.

		<BR>
		<BR>
		On Explorer, you can also use SIL Doulos IPA93 (which is a part of the freely downloadable <A HREF="http://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&id=encore-ipa">SIL Encore</A> package),
		but this font doesn't display as well as <A HREF="http://scripts.sil.org/cms/scripts/page.php?site_id=nrsi&item_id=DoulosSILfont">Doulos SIL</A>.<BR>
		<BR>
		Once a phonetic font is correctly installed on your system, set CCamelOT to use it by going to the <A HREF="font.cgi">font page</A>.<BR>
		<BR>
	</DIV>

	<A HREF="javascript:toggleView('code')">Can I see/use the code?</A><BR>
	<BR>
	<DIV CLASS="helpDiv" ID="code">
		You can find the code, and ideas for what to do with it, in the <A HREf="needCON.cgi#myself">I need a constraint!</A> page.<BR>
		<BR>
	</DIV>

	

	<A HREF="javascript:toggleView('Candidates')">Prosodic structure and syllabification</A><BR>
	<BR>
	<DIV CLASS="helpDiv" ID="Candidates">
		The inputs are made out of segements, moras, and syllables. Consonants can have 0-2 moras
		underlyingly; vowels can have 1-3. Syllables are absent from the input by fiat,
		so all non-moraic consonants start the derivation as "appendix" consonants
		(i.e. not dominated by a syllable).<BR>
		<BR>
		Syllabification is done with complete faithfulness to segments and moras.
		No moras are ever deleted or epenthesized. However, consonants may change
		their affiliations to moras they don't head.
		Syllabification proceeds in three steps:
		<OL>
			<LI>A syllable is built on top of every moraic segment.<BR><BR>
			<LI>
				If two syllables are adjacent (no intervening segments),
				and the second syllable is mono-moraic, and its nucleus is
				less sonorous than the preceding nucleus, then the two syllables
				are merged &mdash; so long as this merger improves harmony.
				This is done recursively, left to right.<BR><BR>
			<LI>
				Syllables look  for onsets and codas: each syllable looks to its left and to
				its right for a single consonant that is less sonorous than the segment the syllable
				currently has at the relevant edge. Then, the most harmonic of three scenarios is chosen:
				The consonant stays unsyllabified, the consonant is syllabified as an onset,
				or the consonant is syllabified as a coda.

		</OL>
		Note that syllables don't have units like "Onset", "Nucleus" or "Coda".
		Constraints on syllable structure use the following definitions:<BR>
		<OL>
			<LI>Onset segments are defined as the consonant(s) leading up to the nucleus.<BR><BR>
			<LI>The nucleus is defined as the most sonorous segment in the syllable.<BR><BR>
			<LI>Coda segments are defined as the consonant(s) following the nucleus.<BR><BR>
		</OL>
		Similarly, moras don't have a designated "head". The most sonorous segment dominated by a mora is that mora's head.<BR><BR>
	</DIV>

	<A HREF="javascript:toggleView('Chains')">Chains and repairs</A><BR>
	<BR>
	<DIV CLASS="helpDiv" ID="Chains">
		The derivation starts with the creation of a chain that only contains one link - the faithful candidate. The faithful candidate is created by running the input through the grammar without allowing Faithfulness violations.<BR>
		<BR>
		Next, all the repairs are applied to the faithful candidate. After each repair is performed, the output is syllabified and evaluated by the grammar. If the output is more harmonic than the faithful candidate, a new chain is created that contains two links: the faithful candidate and the newly created form.<BR>
		<BR>
		After all the repairs were applied and zero or more two-link chains were created, the process is repeated for each of the two-link chains. The last link in each chain is subjected to all the repairs, and harmonically improving outputs are put in new three-link chains. This process is repeated until chains can't get any longer.<BR>
		<BR>
		Currently available repairs are:
		<OL>
			<LI>Deletion of a segment with all the moras it heads.<BR>
				Epenthetic segments do not get deleted. Deleting an epenthetic segment will undo a LUM, creating an illict chain.<BR><BR>
			<LI>Epenthesis of a segment (currently either a glottal stop or a schwa) with a mora.<BR><BR>
			<LI>Epenthesis of a segment (currently either a glottal stop or a schwa) without a mora.<BR><BR>
			<LI>Epenthesis of a mora onto an existing segment (no floating moras are created).<BR><BR>
			<LI>Deletion of a mora.<BR><BR>
			<LI>Change of the feature [voice] (from + to &ndash; and vice versa).<BR><BR>
			<LI>Change of the feature [back] (from + to &ndash; and vice versa).<BR><BR>
			<LI>Change of the feature [high] (from + to &ndash; and vice versa).<BR><BR>
			<LI>Change of the feature [low] (from + to &ndash; and vice versa).<BR><BR>
			
		</OL>
		When a feature such as voice is changed, CCamelOT goes through the list of existing
		phonemes, and looks for a phoneme that matches the input phoneme in all features
		except the changed one. If no such phoneme is found (for instance, nasals don't
		have voiceless counterpart), the feature change does not go through.
		<BR>
		<BR>
	</DIV>


	<A HREF="javascript:toggleView('Segments')">Segments, sonority and features</A><BR>
	<BR>
	<DIV CLASS="helpDiv" ID="Segments">
	
		Segments have the following attributes:
		<OL>
			<LI>A sonority level between one and 16, following <A HREF="http://glsa.hypermart.net/cgi-bin/list.cgi?DISS%PARK02%5%N">Parker (2002)</A>. <A HREF="javascript:toggleView('sonority')">List</A>
				<DIV CLASS="helpDiv" ID="sonority">
				<BR>
				<TABLE BORDER=1 CELLSPACING=1 CELLPADDING=6>
					<TR><TD>Low vowels</TD><TD>16</TD></TR>
					<TR><TD>Mid vowels</TD><TD>15</TD></TR>
					<TR><TD>High vowels</TD><TD>14</TD></TR>
					<TR><TD>Schwa</TD><TD>13</TD></TR>
					<TR><TD>Barred i</TD><TD>12</TD></TR>
					<TR><TD>Glides</TD><TD>11</TD></TR>
					<TR><TD>r</TD><TD>10</TD></TR>
					<TR><TD>l</TD><TD>9</TD></TR>
					<TR><TD>(not implemented)</TD><TD>8</TD></TR>
					<TR><TD>(not implemented)</TD><TD>7</TD></TR>
					<TR><TD>Nasals</TD><TD>6</TD></TR>
					<TR><TD>h</TD><TD>5</TD></TR>
					<TR><TD>Voiced fricatives</TD><TD>4</TD></TR>
					<TR><TD>Voiceless fricatives</TD><TD>3</TD></TR>
					<TR><TD>Voiced stops</TD><TD>2</TD></TR>
					<TR><TD>Voiceless stops</TD><TD>1</TD></TR>
				</TABLE>
				</DIV>
				<BR><BR>
			<LI>A place of articulation: LAB, COR and DOR. Vowels are placeless.<BR><BR>
			<LI>The following binary features: 	consonantal, anterior, distributed, lateral, strident, sonorant, continuant, voice, constricted glottis, spread glottis, high, back, low, round, ATR, nasal.<BR><BR>
		</OL>
		This follows SPE pretty closely. Users are welcome to change the features
		any way they want (but note that some constraints may require adjusting too).
		To update phonemes and features, you have to download CCamelOT and run it on your own computer. If you have done this, follow these steps:
		<OL>
			<LI>Use Excel to update the file "Phoneme.xls".
			<LI>Save "Phoneme.xls" as a tab-delimited text file, and call it "Phoneme.txt".
			<LI>Use a text editor to replace all triple double quotes (""") with single double quotes (").
			<LI>Run the file "Phoneme_generator.pl".
		</OL>
	</DIV>

	<A HREF="javascript:toggleView('Constraints')">Constraints</A><BR>
	<BR>
	<DIV CLASS="helpDiv" ID="Constraints">
		The following constraints are currently implemented in CCamelOT:<BR><BR>
		<OL>
EOF

		for (sort keys %{CON()}) {
			if (exists ${CON()}{$_}) {
				print "<LI>" , ${CON()}{$_}->();
				print "<BR>" , ${CONdescription()}{$_} ;
				print "<BR><BR>";
			}
		}

print <<"EOF";
		</OL>
	</DIV>
</DIV>

EOF

# read footer
open (FTR, "<footer.txt") or die "Can't read footer. $!";
print while (<FTR>);
close(FTR);

print <<"EOF";


</BODY>
</HTML>

EOF
