#!/usr/bin/perl -w

use strict;
use warnings;

use CGI;
my $q = new CGI;
use CGI::Cookie;
use URI::Escape;

use Fcntl qw(:flock);

use Phoneme;
use Link;
use MakeHTML;
use Constraints;


# read cookies
my %cookies = fetch CGI::Cookie;

# sets phonetic alphabet
my $alphabet = $cookies{alphabet} || "";
$alphabet = ($alphabet =~ /^alphabet=([^;]*);/)[0] || "";
$alphabet = "APAU" unless ($alphabet =~ /^[IA]PA[U]?$/);
sub unicode { return (index($alphabet,"U")!=-1) ? "U" : ""; }
setAlphabet($alphabet);

# read CON
my $CON = uri_unescape(($cookies{CON} =~ /^CON=([^;]*);/)[0]) || "";

# read inputs
my $inputs = uri_unescape(($cookies{inputs} =~ /^inputs=([^;]*);/)[0]) || "";



my $foundPost = 0;
my $postMsg = "";
my $postErr = "";
my $visibility = 0;

if ($q->param("userName") or $q->param("userAffiliation") or $q->param("userEmail") or $q->param("CON") or $q->param("inputs") or $q->param("content") or $q->param("visibility") or $q->param("comment")) {

	$foundPost = 1;
	
	die "The program aborted because of unexpected characters in the user name field.        $!" if (($q->param("userName")        =~ /[\{\};]/) or (length($q->param("userName"))>60));
	die "The program aborted because of unexpected characters in the user affiliation field. $!" if (($q->param("userAffiliation") =~ /[\{\};]/) or (length($q->param("userAffiliation"))>60));
	die "The program aborted because of unexpected characters in the user email field.       $!" if (($q->param("userEmail")       =~ /[\{\};]/) or (length($q->param("userEmail"))>60));
	die "The program aborted because of unexpected characters in the CON field.              $!" if (($q->param("CON")             =~ /[\{\};]/) or (length($q->param("CON"))>1000));
	die "The program aborted because of unexpected characters in the inputs field.           $!" if (($q->param("inputs")          =~ /[\{\};]/) or (length($q->param("inputs"))>1000));
	die "The program aborted because of unexpected characters in the content field.          $!" if ($q->param("content")    !~ /(ranking|both)/);
	die "The program aborted because of unexpected characters in the visibility field.       $!" if ($q->param("visibility") !~ /(private|public)/);
	die "The program aborted because of unexpected characters in the comment field.          $!" if ($q->param("comment")    =~ /[\{\};]/);
	die "The program aborted because the comment field was too long.                         $!" if (length($q->param("comment"))>2000);
	
	my $userName        = $q->param("userName")        || "";
	my $userAffiliation = $q->param("userAffiliation") || "";
	my $userEmail       = $q->param("userEmail")       || "";
	my $CON             = $q->param("CON")             || "";
	my $inputs          = $q->param("inputs")          || "";
	   $visibility      = $q->param("visibility")      || "";
	my $comment         = $q->param("comment")         || "";
	$comment =~ s/\n/<BR>/g;
	
	$postErr = "Posting could not be completed because data was missing (.vis=$visibility) "  unless ($visibility eq "private" or $visibility eq "public");
	$postErr = "Posting could not be completed because data was missing (.usnm)" if ($userName eq "");
	$postErr = "Posting could not be completed because data was missing (.usem)" if ($userEmail eq "");
	$postErr = "Posting could not be completed because data was missing (.CON)"  if ($CON eq "");

	if ($postErr eq "") {
		
		$visibility = ($visibility eq "public") ? 1 : 0 ;
		my $time = localtime ;

		# read id's and generate a new unique id
		my $CONid = 0;
		open (CONS, "<data/CONs.txt") or die "Can't read CONs.txt $!";
		flock(CONS, LOCK_SH);
		while (<CONS>) {
			my $id = (/^id=(\d+);/)[0];
			$CONid = $id if ($CONid<$id); 
		}
		close(CONS);
		$CONid++;
		
		#write record to file
		my $postStr = "id=$CONid;public=$visibility;email=$userEmail;name=$userName;affiliation=$userAffiliation;CON=$CON;inputs=$inputs;comment=$comment;time=$time;\n";
		open (CONS, ">>data/CONs.txt") or die "Can't write to CONs.txt $!";
		flock(CONS, LOCK_EX);
		print CONS $postStr;
		close(CONS);
	}
}



#make sure there is a CON to save
my $errorMessage = "";
if ($CON eq "") {
	$errorMessage = "No constraint hierarchy to save.";
}


# make string representation of CON
my $CONSeparator = "<SPAN STYLE='font-size:1px;'> </SPAN><SPAN CLASS='CONsep'><SPAN STYLE='position:relative;left:3px;'>></SPAN><SPAN STYLE='position:relative;left:-3px;'>></SPAN></SPAN>";
my $showCON;
my @currCON = split(/,/,$CON);
for (@currCON) {
	if (exists ${CON()}{$_}) {
		$_ = ${CON()}{$_}->();
	} else {
		die "The constraint named '$_' caused an error. $!"
	}
}
$showCON = join($CONSeparator, @currCON);


#read current user
my $userEmail = uri_unescape(($cookies{userEmail} =~ /^userEmail=([^;]*);/)[0]) || "";
my $obfuscatedEmail = "";
my $userName = "";
my $userAffiliation = "";

if ($userEmail eq "") {
	print $q->redirect(-uri=>'login.cgi');
} else {
	if ($errorMessage eq "") {
		# read user's name and affiliation
		open (USERS, "<data/users.cgi") or die "Can't read user information. $!";
		flock(USERS, LOCK_SH);
		while (<USERS>) {
			if (index($_,$userEmail.";")==0) {
				$userName       = uri_unescape((/;name=([^;]+);/)[0]);
				$userAffiliation= uri_unescape((/;affiliation=([^;]+);/)[0]) || "";
				last;
			}
		}
		close(USERS);
		if ($userName eq "") {
			$errorMessage = "Your user information could not be found. Try logging in again.";
		}
	}


	# obfuscate $userEmail
	my $link = $userEmail;
	$obfuscatedEmail = "<SCRIPT type='text/javascript' language='javascript'>\n//<![CDATA[\n	document.write(String.fromCharCode(";
	for (my $i=0; $i<length($link); $i++) {
		$obfuscatedEmail .= ord(substr($link,$i,1));
		$obfuscatedEmail .= ($i<(length($link)-1)) ? "," : ")";
	}
	$obfuscatedEmail .= ");\n	document.close();\n//]]>\n</SCRIPT>\n";

}








# create the body of the page
print $q->header;
print <<"EOF";
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<HTML xmlns="http://www.w3.org/1999/xhtml" lang="en-US" xml:lang="en-US">
<HEAD>
	<TITLE>CCamelOT - An OT-CC implementation</TITLE>
	
	<LINK rel="stylesheet" type="text/css" href="mich.css" />
	<META http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
	<META http-equiv="Pragma" content="no-cache" />
	
	<SCRIPT src="toolbox.js" type="text/javascript"></SCRIPT>
	
	<SCRIPT type="text/javascript">
		function init() {
			document.getElementById("ranking_tab").className = "navOn";
			document.getElementById("saveTab").className = "menuOn" ;
		}
	</SCRIPT>

</HEAD>

<BODY LEFTMARGIN=0 TOPMARGIN=0 MARGINWIDTH=0 MARGINHEIGHT=0 onLoad="init();">
<FORM NAME="save" METHOD="post" ACTION="save.cgi">
<INPUT TYPE=hidden NAME="userName"        VALUE="$userName">
<INPUT TYPE=hidden NAME="userAffiliation" VALUE="$userAffiliation">
<INPUT TYPE=hidden NAME="userEmail"       VALUE="$userEmail">
<INPUT TYPE=hidden NAME="CON"             VALUE="$CON">
<INPUT TYPE=hidden NAME="inputs"          VALUE="$inputs">

EOF

# read nav bar
open (NAV, "<navbar.txt") or die "Can't read navbar. $!";
print while (<NAV>);
close(NAV);

my $menuStr;
open (NAV, "<ranking_menu.txt") or die "Can't read ranking menu. $!";
print while (<NAV>);
close(NAV);

print "<DIV CLASS='menuMain'>";

if ($foundPost) {

	if ($postErr) {
		print "<BR><BR><DIV CLASS='errorMsg'>$postErr</DIV>\n";
	} else {
		print <<"		EOF";
			<BR>			
			Your constraint ranking was saved.<BR><BR>
			You can find it in the <A HREF='CONs.cgi'>Open</A> tab, under
		EOF
		($visibility) ? print '"Public' : print '"Private';
		print " rankings\".<BR><BR><BR><BR><BR><BR></TD></TR></TABLE><BR>";
	}


} else {

	if ($errorMessage) {

		print "<BR><DIV CLASS='errorMsg'>$errorMessage</DIV><BR>";

	} else {
        
		print "$showCON<BR><BR>";

		if ($inputs eq "" or $inputs eq "::") {
			print "What would you like to save?<BR>";
			print "<INPUT TYPE=radio NAME='content' VALUE='ranking' CHECKED> Just the constraint ranking above<BR>";
			print "<INPUT TYPE=radio NAME='content' VALUE='both' DISABLED> <SPAN STYLE='color:#999999;'> The constraint ranking above together with the following input</SPAN><BR>";
		} else {
			print "\t\tWhat would you like to save?<BR>";
			print "<INPUT TYPE=radio NAME='content' VALUE='ranking'> Just the constraint ranking above<BR>";
			print "<INPUT TYPE=radio NAME='content' VALUE='both' CHECKED> The constraint ranking above together with the following input: <BR><BR>";
			print "<DIV STYLE='font-size:150%;line-size:150%'>&nbsp; &nbsp; " , MakeHTML::candHTML(Link->new(split(/:/,$inputs)),1) , "</DIV>\n";
		}

		print <<"		EOF";
			<BR>
			How would you like to save this?<BR>
			<INPUT TYPE=radio NAME='visibility' VALUE='private'> Privately (visible to you)<BR>
			<INPUT TYPE=radio NAME='visibility' VALUE='public' CHECKED> Publicly (visible to all)<BR>
			<BR>
			Comment:<BR>
			<TEXTAREA NAME='comment' CLASS='textInput' STYLE='width:30em;height:6em;' ROWS=4 COLS=40 MAXLENGTH=200></TEXTAREA><BR>
			<BR>
			The following details will be used:<BR>
			Your email: $obfuscatedEmail<BR>
			Your name: $userName<BR>
		EOF
   
		print "Your affiliation: $userAffiliation <BR>\n" if ($userAffiliation);
		
		print <<"		EOF";
			<BR>
			<INPUT CLASS='controlButton' TYPE=submit VALUE='   Save   ' onClick='return checkForm([this.form.comment])'>
			<DIV CLASS='regInstructions'>&nbsp;<BR>The eMail Obfuscator by Tim Williams is used in this page and all other pages where emails are shown.</DIV>
			</TD>
		</TR>
		</TABLE>
		<BR>
		EOF
	}
}

print "</DIV>";

# read footer
open (FTR, "<footer.txt") or die "Can't read footer. $!";
print while (<FTR>);
close(FTR);


print <<"EOF";

</FORM>

</BODY>
</HTML>

EOF

